#!/usr/bin/env python
# curl -i http://localhost:8000/foo/
from circuits.web import Controller, Server
from circuits.web.wsgi import Gateway


def foo(environ, start_response):
    start_response('200 OK', [('Content-Type', 'text/plain')])
    return ['Foo!']


class Root(Controller):
    """App Rot"""

    def index(self):
        return 'Hello World!'


app = Server(('0.0.0.0', 8000))
Root().register(app)
Gateway({'/foo': foo}).register(app)
app.run()



#!/usr/bin/env python
import pytest

from circuits import Component, Event, handler


class wait(Event):
    """wait Event"""

    success = True


class hello(Event):
    """hello Event"""

    success = True


class App(Component):
    @handler('wait')
    def _on_wait(self):
        e = hello()
        x = self.fire(e)
        yield self.wait(e)
        yield x.value

    def hello(self):
        return 'Hello World!'


@pytest.fixture()
def app(request, manager, watcher):
    app = App().register(manager)
    assert watcher.wait('registered')

    def finalizer():
        app.unregister()

    request.addfinalizer(finalizer)

    return app


def test_wait_instance(manager, watcher, app):
    x = manager.fire(wait())
    assert watcher.wait('wait_success')

    value = x.value
    assert value == 'Hello World!'



#!/usr/bin/env python
from circuits import Component, Debugger, handler
from circuits.web import Controller, Server
from circuits.web.errors import forbidden


class ACL(Component):
    allowed = ['127.0.0.1']

    @handler('request', priority=1.0)
    def on_request(self, event, request, response):
        """
        Filter Requests applying IP based Authorization

        Filter any incoming requests at a higher priority than the
        default dispatcher and apply IP based Authorization returning
        a 403 Forbidden response if the Remote IP Address does not
        match the allowed set.
        """
        if request.remote.ip not in self.allowed:
            event.stop()
            return forbidden(request, response)
        return None


class Root(Controller):
    def index(self):
        return 'Hello World!'


app = Server(('0.0.0.0', 8000))
ACL().register(app)
Root().register(app)
Debugger().register(app)
app.run()



#!/usr/bin/env python
import os
import sqlite3

import macros
from creoleparser import Parser, create_dialect, creole11_base

import circuits
from circuits.web import Controller, Logger, Server, Static


text2html = Parser(
    create_dialect(creole11_base, macro_func=macros.dispatcher),
    method='xhtml',
)


class Wiki:
    def __init__(self, database):
        super().__init__()

        create = not os.path.exists(database)

        self._cx = sqlite3.connect(database)
        self._cu = self._cx.cursor()

        if create:
            self._cu.execute('CREATE TABLE pages (name, text)')
            for defaultpage in os.listdir('defaultpages'):
                filename = os.path.join('defaultpages', defaultpage)
                self.save(defaultpage, open(filename).read())

    def save(self, name, text):
        self._cu.execute('SELECT COUNT() FROM pages WHERE name=?', (name,))
        row = self._cu.fetchone()
        if row[0]:
            self._cu.execute(
                'UPDATE pages SET text=? WHERE name=?',
                (text, name),
            )
        else:
            self._cu.execute(
                'INSERT INTO pages (name, text) VALUES (?, ?)',
                (name, text),
            )
        self._cx.commit()

    def get(self, name, default=None):
        self._cu.execute('SELECT text FROM pages WHERE name=?', (name,))
        row = self._cu.fetchone()
        return row[0] if row else default


class Root(Controller):
    db = Wiki('wiki.db')

    environ = {'db': db, 'macros': macros.loadMacros()}

    def GET(self, name='FrontPage', action='view'):
        environ = self.environ.copy()
        environ['page.name'] = name
        environ['parser'] = text2html

        d = {}
        d['title'] = name
        d['version'] = circuits.__version__
        d['menu'] = text2html(self.db.get('SiteMenu', ''), environ=environ)

        text = self.db.get(name, '')
        s = open('tpl/%s.html' % action).read()

        if action == 'view':
            d['text'] = text2html(text, environ=environ)
        else:
            d['text'] = text

        return s % d

    def POST(self, name='FrontPage', **form):
        self.db.save(name, form.get('text', ''))
        return self.redirect(name)


app = Server(('0.0.0.0', 8000))
Static(docroot='static').register(app)
Root().register(app)
Logger().register(app)
app.run()



"""Internet Relay Chat Protocol commands"""

from .events import request
from .message import Message


def AWAY(message=None):
    return request(Message('AWAY', message))


def NICK(nickname, hopcount=None):
    return request(Message('NICK', nickname, hopcount))


def USER(user, host, server, name):
    return request(Message('USER', user, host, server, name))


def PASS(password):
    return request(Message('PASS', password))


def PONG(daemon1, daemon2=None):
    return request(Message('PONG', daemon1, daemon2))


def QUIT(message=None):
    return request(Message('QUIT', message))


def JOIN(channels, keys=None):
    return request(Message('JOIN', channels, keys))


def PART(channels, message=None):
    return request(Message('PART', channels, message))


def PRIVMSG(receivers, message):
    return request(Message('PRIVMSG', receivers, message))


def NOTICE(receivers, message):
    return request(Message('NOTICE', receivers, message))


def KICK(channel, user, comment=None):
    return request(Message('KICK', channel, user, comment))


def TOPIC(channel, topic=None):
    return request(Message('TOPIC', channel, topic))


def MODE(target, *args):
    return request(Message('MODE', target, *args))


def INVITE(nickname, channel):
    return request(Message('INVITE', nickname, channel))


def NAMES(channels=None):
    return request(Message('NAMES', channels))


def WHOIS(nickmasks, server=None):
    return request(Message(server, nickmasks))


def WHO(name=None, o=None):
    return request(Message('WHO', name, o))



#!/usr/bin/env python
"""
Circuits IRC Client

A circuits based IRC Client demonstrating integration with urwid - a curses
application development library and interacting with and processing irc
events from an IRC server.

NB: This is not a full featured client.

For usage type:

   ./circ.py --help
"""

import os
import sys
from optparse import OptionParser
from re import compile as compile_regex
from select import select
from socket import gethostname

from urwid import AttrWrap, Edit, Frame, ListBox, Pile, SimpleListWalker, Text
from urwid.raw_display import Screen

from circuits import Component, __version__ as systemVersion, handler
from circuits.net.sockets import TCPClient, connect
from circuits.protocols.irc import (
    ERR_NICKNAMEINUSE,
    ERR_NOMOTD,
    IRC,
    JOIN,
    NICK,
    PART,
    PRIVMSG,
    QUIT,
    RPL_ENDOFMOTD,
    USER,
    Message,
    request,
)
from circuits.tools import getargspec


USAGE = '%prog [options] host [port]'
VERSION = '%prog v' + systemVersion

MAIN_TITLE = f'cIRC - {systemVersion:s}'

HELP_STRINGS = {
    'main': 'For help, type: /help',
}

CMD_REGEX = compile_regex(
    r'\/(?P<command>[a-z]+) ?' '(?P<args>.*)(?iu)',
)


def back_merge(line, n, t=' '):
    return line[:-n].extend([t.join(line[-n:])])


def parse_options():
    parser = OptionParser(usage=USAGE, version=VERSION)

    parser.add_option(
        '-c',
        '--channel',
        action='store',
        default='#circuits',
        dest='channel',
        help='Channel to join',
    )

    parser.add_option(
        '',
        '--debug',
        action='store_true',
        default=False,
        dest='debug',
        help='Enable debug mode',
    )

    parser.add_option(
        '-n',
        '--nick',
        action='store',
        default=os.environ['USER'],
        dest='nick',
        help='Nickname to use',
    )

    opts, args = parser.parse_args()

    if len(args) < 1:
        parser.print_help()
        raise SystemExit(1)

    return opts, args


class Client(Component):
    channel = 'client'

    def init(self, host, port=6667, opts=None):
        self.host = host
        self.port = port
        self.opts = opts
        self.hostname = gethostname()

        self.nick = opts.nick
        self.ircchannel = opts.channel

        # Add TCPClient and IRC to the system.
        TCPClient(channel=self.channel).register(self)
        IRC(channel=self.channel).register(self)

        self.create_interface()

    def create_interface(self):
        self.screen = Screen()
        self.screen.start()

        self.screen.register_palette(
            [('title', 'white', 'dark blue', 'standout'), ('line', 'light gray', 'black'), ('help', 'white', 'dark blue')],
        )

        self.body = ListBox(SimpleListWalker([]))
        self.lines = self.body.body

        self.title = Text(MAIN_TITLE)
        self.header = AttrWrap(self.title, 'title')

        self.help = AttrWrap(
            Text(HELP_STRINGS['main']),
            'help',
        )

        self.input = Edit(caption='%s> ' % self.ircchannel)
        self.footer = Pile([self.help, self.input])

        self.top = Frame(self.body, self.header, self.footer)

    def ready(self, component):
        """
        Ready Event

        This event is triggered by the underlying ``TCPClient`` Component
        when it is ready to start making a new connection.
        """
        self.fire(connect(self.host, self.port))

    def connected(self, host, port):
        """
        connected Event

        This event is triggered by the underlying ``TCPClient`` Component
        when a successfully connection has been made.
        """
        nick = self.nick
        hostname = self.hostname
        name = f'{nick} on {hostname} using circuits/{systemVersion}'

        self.fire(NICK(nick))
        self.fire(USER(nick, hostname, host, name))

    def numeric(self, source, numeric, *args):
        """
        Numeric Event

        This event is triggered by the ``IRC`` Protocol Component when we have
        received an IRC Numberic Event from server we are connected to.
        """
        if numeric == ERR_NICKNAMEINUSE:
            self.fire(NICK(f'{args[0]:s}_'))
        elif numeric in (RPL_ENDOFMOTD, ERR_NOMOTD):
            self.fire(JOIN(self.ircchannel))

    @handler('stopped', channel='*')
    def _on_stopped(self, component):
        self.screen.stop()

    @handler('generate_events')
    def _on_generate_events(self, event):
        event.reduce_time_left(0)

        size = self.screen.get_cols_rows()

        if select(self.screen.get_input_descriptors(), [], [], 0.1)[0] != []:
            _timeout, keys, _raw = self.screen.get_input_nonblocking()

            for k in keys:
                if k == 'window resize':
                    size = self.screen.get_cols_rows()
                    continue
                if k == 'enter':
                    self.processCommand(self.input.get_edit_text())
                    self.input.set_edit_text('')
                    continue

                self.top.keypress(size, k)
                self.input.set_edit_text(self.input.get_edit_text() + k)

        self.update_screen(size)

    def unknownCommand(self, command):
        self.lines.append(Text('Unknown command: %s' % command))

    def syntaxError(self, command, args, expected):
        self.lines.append(
            Text(
                f'Syntax error ({command:s}): {args:s} Expected: {expected:s}',
            ),
        )

    def processCommand(self, s):  # noqa
        match = CMD_REGEX.match(s)
        if match is not None:
            command = match.groupdict()['command']
            tokens = match.groupdict()['args'].split(' ') if match.groupdict()['args'] != '' else []

            fn = 'cmd' + command.upper()
            if hasattr(self, fn):
                f = getattr(self, fn)
                if callable(f):
                    args, vargs, _kwargs, default = getargspec(f)
                    args.remove('self')
                    if len(args) == len(tokens):
                        if len(args) == 0:
                            f()
                        else:
                            f(*tokens)
                    else:
                        if len(tokens) > len(args):
                            if vargs is None:
                                if len(args) > 0:
                                    factor = len(tokens) - len(args) + 1
                                    f(*back_merge(tokens, factor))
                                else:
                                    self.syntaxError(
                                        command,
                                        ' '.join(tokens),
                                        ' '.join(x for x in args + [vargs] if x is not None),
                                    )
                            else:
                                f(*tokens)
                        elif default is not None and len(args) == (len(tokens) + len(default)):
                            f(*(tokens + list(default)))
                        else:
                            self.syntaxError(
                                command,
                                ' '.join(tokens),
                                ' '.join(x for x in args + [vargs] if x is not None),
                            )
        else:
            if self.ircchannel is not None:
                self.lines.append(Text(f'<{self.nick}> {s}'))
                self.fire(PRIVMSG(self.ircchannel, s))
            else:
                self.lines.append(Text('No channel joined. Try /join #<channel>'))

    def cmdEXIT(self, message=''):
        self.fire(QUIT(message))
        raise SystemExit(0)

    def cmdSERVER(self, host, port=6667):
        self.fire(connect(host, port))

    def cmdSSLSERVER(self, host, port=6697):
        self.fire(connect(host, port, secure=True))

    def cmdJOIN(self, channel):
        if self.ircchannel is not None:
            self.cmdPART(self.ircchannel, 'Joining %s' % channel)
        self.fire(JOIN(channel))
        self.ircchannel = channel

    def cmdPART(self, channel=None, message='Leaving'):
        if channel is None:
            channel = self.ircchannel
        if channel is not None:
            self.fire(PART(channel, message))
            self.ircchannel = None

    def cmdQUOTE(self, message):
        self.fire(request(Message(message)))

    def cmdQUIT(self, message='Bye'):
        self.fire(QUIT(message))

    def update_screen(self, size):
        canvas = self.top.render(size, focus=True)
        self.screen.draw_screen(size, canvas)

    @handler('notice', 'privmsg')
    def _on_notice_or_privmsg(self, event, source, target, message):
        nick, _ident, _host = source

        if event.name == 'notice':
            self.lines.append(Text(f'-{nick}- {message}'))
        else:
            self.lines.append(Text(f'<{nick}> {message}'))


def main():
    opts, args = parse_options()

    host = args[0]
    port = int(args[1]) if len(args) > 1 else 6667

    # Configure and run the system.

    client = Client(host, port, opts=opts)

    if opts.debug:
        from circuits import Debugger

        Debugger(file=sys.stderr).register(client)

    client.run()


if __name__ == '__main__':
    main()



#!/usr/bin/env python
from circuits.web import Controller
from circuits.web.client import Client, request


class Root(Controller):
    def index(self):
        return 'Hello World!'


def test(webapp):
    client = Client()
    client.start()

    client.fire(request('GET', webapp.server.http.base))
    while client.response is None:
        pass

    client.stop()

    response = client.response
    assert response.status == 200
    assert response.reason == 'OK'

    s = response.read()
    assert s == b'Hello World!'



"""Event Tests"""

import pytest

from circuits import Component, Event


class test(Event):
    """test Event"""


class App(Component):
    def test(self):
        return 'Hello World!'


def test_repr():
    app = App()
    while len(app):
        app.flush()

    e = test()

    s = repr(e)
    assert s == '<test[] ( )>'

    app.fire(e)

    s = repr(e)
    assert s == '<test[*] ( )>'


def test_create():
    app = App()
    while len(app):
        app.flush()

    e = Event.create('test')

    s = repr(e)
    assert s == '<test[] ( )>'

    app.fire(e)

    s = repr(e)
    assert s == '<test[*] ( )>'


def test_getitem():
    app = App()
    while len(app):
        app.flush()

    e = test(1, 2, 3, foo='bar')

    assert e[0] == 1
    assert e['foo'] == 'bar'

    def f(e, k):
        return e[k]

    pytest.raises(TypeError, f, e, None)


def test_setitem():
    app = App()
    while len(app):
        app.flush()

    e = test(1, 2, 3, foo='bar')

    assert e[0] == 1
    assert e['foo'] == 'bar'

    e[0] = 0
    e['foo'] = 'Hello'

    def f(e, k, v):
        e[k] = v

    pytest.raises(TypeError, f, e, None, None)

    assert e[0] == 0
    assert e['foo'] == 'Hello'


def test_subclass_looses_properties():
    class hello(Event):
        success = True

    e = hello().child('success')
    assert e.success is False



from time import sleep

import requests

from circuits import Component, Debugger, Event, Timer, Worker, task


def download_web_page(url):
    print(f'Downloading {url}')
    response = requests.get(url)
    sleep(2)  # This website is really slow.
    # Only returning portion of web page.
    # You would probably process web page for data before sending back
    return response.text[:200]


class App(Component):
    def init(self, *args, **kwargs):
        self.foo_count = 0
        Worker(process=False).register(self)

    def foo(self):
        self.foo_count += 1
        print('Foo!')
        if self.foo_count > 10:
            self.stop()

    def started(self, component):
        # x = yield self.call(task(factorial, 10))
        Timer(1, Event.create('foo'), persist=True).register(self)
        self.fire(task(download_web_page, 'http://www.slickdeals.net'))  # async
        self.fire(task(download_web_page, 'http://www.google.com'))  # async
        self.fire(task(download_web_page, 'http://www.yahoo.com'))  # async

    def task_success(self, function_called, function_result):
        _func, url_called = function_called
        print(f'url {url_called} gave {function_result}')


if __name__ == '__main__':
    app = App()
    Debugger().register(app)
    app.run()



#!/usr/bin/env python
# firefox http://127.0.0.1:8000/websocket.html

from circuits import Component, Debugger
from circuits.net.events import write
from circuits.web import Controller, Logger, Server, Static
from circuits.web.dispatchers import WebSocketsDispatcher


class Echo(Component):
    channel = 'wsserver'

    def read(self, sock, data):
        self.fireEvent(write(sock, 'Received: ' + data))


class Root(Controller):
    def index(self):
        return 'Hello World!'


app = Server(('0.0.0.0', 8000))
Debugger().register(app)
Static().register(app)
Echo().register(app)
Root().register(app)
Logger().register(app)
WebSocketsDispatcher('/websocket').register(app)
app.run()



#!/usr/bin/env python
from circuits.core.components import BaseComponent
from circuits.core.handlers import handler

from .helpers import HTTPError, urlopen


class Root(BaseComponent):
    channel = 'web'

    @handler('request', priority=0.2)
    def request(self, request, response):
        raise Exception()


def test(webapp):
    try:
        Root().register(webapp)
        urlopen(webapp.server.http.base)
    except HTTPError as e:
        assert e.code == 500
    else:
        assert False



"""
Include macros

Macros for inclusion of other wiki pages
"""

from genshi import builder


def include(macro, environ, pagename=None, *args, **kwargs):
    """Return the parsed content of the page identified by arg_string"""
    if pagename is None:
        return None

    db = environ['db']
    page = db.get(pagename, None)

    if page is not None:
        environ['page.name'] = pagename

        return environ['parser'].generate(page, environ=environ)
    return None


def include_raw(macro, environ, pagename=None, *args, **kwargs):
    """
    Return the raw text of the page identified by arg_string, rendered
    in a <pre> block.
    """
    if pagename is None:
        return None

    db = environ['db']
    page = db.get(pagename, None)

    if page is not None:
        return builder.tag.pre(page, class_='plain')
    return None


def include_source(macro, environ, pagename=None, *args, **kwargs):
    """
    Return the parsed text of the page identified by arg_string, rendered
    in a <pre> block.
    """
    if pagename is None:
        return None

    db = environ['db']
    page = db.get(pagename, None)

    if page is not None:
        environ['page.name'] = pagename

        return builder.tag.pre(
            environ['parser'].render(page, environ=environ).decode('utf-8'),
        )
    return None



#!/usr/bin/env python
import pytest

from circuits import Component, Event
from circuits.core import Value
from circuits.node.protocol import Protocol
from circuits.node.utils import dump_event, dump_value


pytestmark = pytest.mark.skipif(pytest.PLATFORM == 'win32', reason='Broken on Windows')


class return_value(Event):
    success = True


class firewall_block(Event):
    pass


class AppClient(Component):
    write_data = b''

    def return_value(self):
        return 'Hello server!'

    def write(self, data):
        self.write_data = data


class AppFirewall(Component):
    write_data = b''

    def fw_receive(self, event, sock):
        return self.__event_is_allow(event)

    def fw_send(self, event, sock):
        return self.__event_is_allow(event)

    def write(self, data):
        self.write_data = data

    def __event_is_allow(self, event):
        allow = event.name == 'return_value' and 'prohibits_channel' not in event.channels

        if not allow:
            self.fire(firewall_block())

        return allow


class AppServer(Component):
    write_data = b''
    write_sock = None

    def return_value(self):
        return 'Hello client!'

    def write(self, sock, data):
        self.write_sock = sock
        self.write_data = data


@pytest.fixture()
def app_client(request, manager, watcher):
    app = AppClient()
    app.register(manager)
    watcher.wait('registered')

    app.protocol = Protocol().register(app)
    watcher.wait('registered')

    def finalizer():
        app.unregister()

    request.addfinalizer(finalizer)

    return app


@pytest.fixture()
def app_firewall(request, manager, watcher):
    app = AppFirewall()
    app.register(manager)
    watcher.wait('registered')

    app.protocol = Protocol(
        sock='sock obj',
        receive_event_firewall=app.fw_receive,
        send_event_firewall=app.fw_send,
    ).register(app)
    watcher.wait('registered')

    def finalizer():
        app.unregister()

    request.addfinalizer(finalizer)

    return app


@pytest.fixture()
def app_server(request, manager, watcher):
    app = AppServer()
    app.register(manager)
    watcher.wait('registered')

    app.protocol = Protocol(sock='sock obj', server=True).register(app)
    watcher.wait('registered')

    def finalizer():
        app.unregister()

    request.addfinalizer(finalizer)

    return app


def test_add_buffer(app_client, watcher):
    packet = str.encode(dump_event(return_value(), 1))
    app_client.protocol.add_buffer(packet)
    assert watcher.wait('return_value_success')
    assert watcher.wait('write')

    value = Value()
    value.value = 'Hello server!'
    value.errors = False
    value.node_call_id = 1
    assert app_client.write_data == str.encode(dump_value(value) + '~~~')


def test_add_buffer_server(app_server, watcher):
    packet = str.encode(dump_event(return_value(), 1))
    app_server.protocol.add_buffer(packet)
    assert watcher.wait('return_value_success')
    assert watcher.wait('write')

    value = Value()
    value.value = 'Hello client!'
    value.errors = False
    value.node_call_id = 1
    assert app_server.write_data == str.encode(dump_value(value) + '~~~')
    assert app_server.write_sock == 'sock obj'


def test_firewall_receive(app_firewall, watcher):
    # good event
    packet = str.encode(dump_event(return_value(), 1))
    app_firewall.protocol.add_buffer(packet)
    assert watcher.wait('return_value')

    # bad name
    packet = str.encode(dump_event(Event.create('unallow_event'), 1))
    app_firewall.protocol.add_buffer(packet)
    assert watcher.wait('firewall_block')

    # bad channel
    event = return_value()
    event.channels = ('prohibits_channel',)
    packet = str.encode(dump_event(event, 1))
    app_firewall.protocol.add_buffer(packet)
    assert watcher.wait('firewall_block')


def test_firewall_send(app_firewall, watcher):
    # good event
    event = return_value()
    generator = app_firewall.protocol.send(event)
    next(generator)  # exec
    assert watcher.wait('write')
    assert app_firewall.write_data == str.encode(dump_event(event, 0) + '~~~')

    # bad name
    generator = app_firewall.protocol.send(Event.create('unallow_event'))
    next(generator)  # exec
    assert watcher.wait('firewall_block')

    # bad channel
    event = return_value()
    event.channels = ('prohibits_channel',)
    generator = app_firewall.protocol.send(event)
    next(generator)  # exec
    assert watcher.wait('firewall_block')


def test_send(app_client, watcher):
    event = return_value()
    generator = app_client.protocol.send(event)
    next(generator)  # exec

    assert watcher.wait('write')
    assert app_client.write_data == str.encode(dump_event(event, 0) + '~~~')

    value = Value()
    value.value = 'Hello server!'
    value.errors = False
    value.node_call_id = 0
    app_client.protocol.add_buffer(str.encode(dump_value(value) + '~~~'))

    assert next(generator).getValue() == value.value


def test_send_server(app_server, watcher):
    event = return_value()
    generator = app_server.protocol.send(event)
    next(generator)  # exec

    assert watcher.wait('write')
    assert app_server.write_data == str.encode(dump_event(event, 0) + '~~~')
    assert app_server.write_sock == 'sock obj'

    value = Value()
    value.value = 'Hello client!'
    value.errors = False
    value.node_call_id = 0
    app_server.protocol.add_buffer(str.encode(dump_value(value) + '~~~'))

    assert next(generator).getValue() == value.value



#!/usr/bin/env python

from subprocess import PIPE, STDOUT, Popen


def test_linkcheck(tmpdir):
    doctrees = tmpdir.join('doctrees')
    htmldir = tmpdir.join('html')
    p = Popen(
        [
            'sphinx-build',
            '-W',
            '-blinkcheck',
            '-d',
            str(doctrees),
            'source',
            str(htmldir),
        ],
        stdout=PIPE,
        stderr=STDOUT,
    )
    stdout, _ = p.communicate()
    if p.wait() != 0:
        print(stdout)


def test_build_docs(tmpdir):
    doctrees = tmpdir.join('doctrees')
    htmldir = tmpdir.join('html')
    p = Popen(
        [
            'sphinx-build',
            '-W',
            '-bhtml',
            '-d',
            str(doctrees),
            'source',
            str(htmldir),
        ],
        stdout=PIPE,
        stderr=STDOUT,
    )
    stdout, _ = p.communicate()
    if p.wait() != 0:
        print(stdout)



#!/usr/bin/env python
"""
DNS Client Example

A simple little DNS Client example using
`dnslib <https://pypi.python.org/pypi/dnslib>`_
to handle the DNS protocol parsing and packet
deconstruction (*a really nice library btw with
great integration into circuits*).

Specify the server, port, and query as arguments
to perform a lookup against a server using UDP.

To run this example::

    pip install dnslib
    ./dnsclient.py 8.8.8.8 53 google.com
"""

import sys

from dnslib import DNSQuestion, DNSRecord

from circuits import Component, Debugger, Event
from circuits.net.events import write
from circuits.net.sockets import UDPClient


class reply(Event):
    """reply Event"""


class DNS(Component):
    """DNS Protocol Handling"""

    def read(self, peer, data):
        self.fire(reply(peer, DNSRecord.parse(data)))


class Dummy(Component):
    """
    A Dummy DNS Handler

    This just parses the reply packet and
    prints any RR records it finds.
    """

    def reply(self, peer, response):
        id = response.header.id
        qname = response.q.qname

        print(
            f'DNS Response from {peer[0]:s}:{peer[1]:d} id={id:d} qname={str(qname):s}',
            file=sys.stderr,
        )

        for rr in response.rr:
            print(f' {rr!s:s}')

        raise SystemExit(0)


class DNSClient(Component):
    """
    DNS Client

    This ties everything together in a nice
    configurable way with protocol, transport,
    and dummy handler as well as optional debugger.
    """

    def init(self, server, port, query, verbose=False):
        self.server = server
        self.port = int(port)
        self.query = query

        if verbose:
            Debugger().register(self)

        self.transport = UDPClient(0).register(self)
        self.protocol = DNS().register(self)
        self.dummy = Dummy().register(self)

    def started(self, manager):
        print('DNS Client Started!', file=sys.stderr)

    def ready(self, client, bind):
        print('Ready! Bound to {:s}:{:d}'.format(*bind), file=sys.stderr)

        request = DNSRecord(q=DNSQuestion(self.query))
        self.fire(write((self.server, self.port), request.pack()))


DNSClient(*sys.argv[1:], verbose=True).run()



import contextlib

from circuits import BaseComponent, handler
from circuits.net.sockets import TCPServer

from .protocol import Protocol


class Server(BaseComponent):
    """Node server."""

    channel = 'node'
    __protocols = {}

    def __init__(
        self, port, server_ip='0.0.0.0', channel=channel, receive_event_firewall=None, send_event_firewall=None, **kwargs
    ):
        """
        Create server on node system.

        :param port:    start server on this port.
        :type port:     int

        :param server_ip:   An optional keyword argument which which define
                            ip where the socket has listen to.
                            **Default:** ``0.0.0.0`` (all ip is allowed)
        :type server_ip:     str

        :param channel: An optional keyword argument which if defined,
                        set channel used for node event. **Default:** ``node``
        :type channel:  str

        :param receive_event_firewall: An optional keyword argument which if
                                       defined, set function or method to call
                                       to check if event is allowed for sending
                                       **Default:** ``None`` (no firewall)
        :type receive_event_firewall:  function
        :type receive_event_firewall:  method

        :param send_event_firewall:  An optional keyword argument which if
                                    defined, set function or method to call to
                                    check if event is allowed for executing
                                    **Default:** ``None`` (no firewall)
        :type send_event_firewall:   function
        :type send_event_firewall:   method
        """
        super().__init__(channel=channel, **kwargs)

        bind = (server_ip, port)
        self.server = TCPServer(bind, channel=self.channel, **kwargs)
        self.server.register(self)
        self.__receive_event_firewall = receive_event_firewall
        self.__send_event_firewall = send_event_firewall

    def send(self, event, sock, no_result=False):
        """
        Send event to peer

        :param event:    Event to execute remotely.
        :type event:     :class:`circuits.core.events.Event`

        :param sock:    Client's socket (peer selection).
        :type sock:     :class:`socket.socket`

        :param no_result:   An optional keyword argument which if True
                            don't return the event result.
                            **Default:** ``False`` (wait the result)
        :type no_result:     bool

        :return: The result of remote event
        :rtype: generator
        """
        iterator = self.__protocols[sock].send(event)
        if no_result:
            event.node_without_result = True
            with contextlib.suppress(StopIteration):
                next(iterator)
        return iterator

    def send_to(self, event, socks):
        """
        Send event to multiple peer

        :param event:    Event to execute remotely.
        :type event:     :class:`circuits.core.events.Event`

        :param socks:    Client's socket list (peer selection).
        :type socks:     list of :class:`socket.socket`
        """
        for sock in socks:
            self.send(event, sock, no_result=True)

    def send_all(self, event):
        """
        Send event to all peer

        :param event:    Event to execute remotely.
        :type event:     :class:`circuits.core.events.Event`
        """
        self.send_to(event, list(self.__protocols))

    @handler('read')
    def _on_read(self, sock, data):
        self.__protocols[sock].add_buffer(data)

    @property
    def host(self):
        if hasattr(self, 'server'):
            return self.server.host
        return None

    @property
    def port(self):
        if hasattr(self, 'server'):
            return self.server.port
        return None

    def get_socks(self):
        """
        Get clients sockets list

        :return: The list of client socket
        :rtype: list of :class:`socket.socket`
        """
        return list(self.__protocols)

    @handler('connect')
    def __connect_peer(self, sock, host, port):
        self.__protocols[sock] = Protocol(
            sock=sock,
            server=self.server,
            receive_event_firewall=self.__receive_event_firewall,
            send_event_firewall=self.__send_event_firewall,
            channel=self.channel,
        ).register(self)

    @handler('disconnect')
    def __disconnect_peer(self, sock):
        for s in self.__protocols.copy():
            try:
                s.getpeername()
            except Exception:
                del self.__protocols[s]



#!/usr/bin/env python
"""
wget Example

A basic wget-like clone that asynchronously connections
a remote web server requesting a given resource.
"""

import sys

from circuits import Component
from circuits.web.client import Client, request


class WebClient(Component):
    def init(self, url):
        self.url = url

        Client().register(self)

    def ready(self, *args):
        self.fire(request('GET', self.url))

    def response(self, response):
        print(f'{response.status:d} {response.reason:s}')
        print(
            '\n'.join(f'{k:s}: {v:s}' for k, v in response.headers.items()),
        )
        print(response.read())
        raise SystemExit(0)


WebClient(sys.argv[1]).run()



#!/usr/bin/env python

import pytest

from circuits import Component, Event


pytestmark = pytest.mark.skip('XXX: This test fails intermittently')


class test(Event):
    """test Event"""


class coroutine1(Event):
    """coroutine Event"""

    complete = True


class coroutine2(Event):
    """coroutine Event"""

    complete = True


class App(Component):
    returned = False

    def test(self, event):
        event.stop()
        return 'Hello World!'

    def coroutine1(self):
        print('coroutine1')
        yield self.call(test())
        print('returned')
        self.returned = True

    def coroutine2(self):
        print('coroutine2')
        self.fire(test())
        yield self.wait('test')
        print('returned')
        self.returned = True


@pytest.fixture()
def app(request, manager, watcher):
    app = App().register(manager)
    assert watcher.wait('registered')

    def finalizer():
        app.unregister()

    request.addfinalizer(finalizer)

    return app


def test_coroutine(manager, watcher, app):
    manager.fire(coroutine1())
    assert watcher.wait('coroutine1_complete')
    assert app.returned, 'coroutine1'

    app.returned = False
    manager.fire(coroutine2())
    assert watcher.wait('coroutine2_complete')
    assert app.returned, 'coroutine2'



#!/usr/bin/env python

"""
Example usage for StompClient component

Requires a STOMP server to connect to.

"""

import logging
import ssl

from circuits import Component, Event, Timer
from circuits.protocols.stomp.client import ACK_AUTO, StompClient
from circuits.protocols.stomp.events import connect, send, subscribe


LOG = logging.getLogger(__name__)


class QueueHandler(Component):
    def __init__(self, queue, host=None, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.queue = queue
        self.host = host

    def registered(self, event, component, parent):
        if component.parent is self:
            self.fire(Event.create('reconnect'))

    def connected(self):
        """Client has connected to the STOMP server"""
        LOG.info('STOMP connected.')
        # Let's subscribe to the message destination
        self.fire(subscribe(self.queue, ack=ACK_AUTO))

    def subscribe_success(self, event, *args, **kwargs):
        """Subscribed to message destination"""
        # Let's fire off some messages
        self.fire(send(headers=None, body='Hello World', destination=self.queue))
        self.fire(send(headers=None, body='Hello Again World', destination=self.queue))

    def heartbeat_timeout(self):
        """Heartbeat timed out from the STOMP server"""
        LOG.error('STOMP heartbeat timeout!')
        # Set a timer to automatically reconnect
        Timer(10, Event.create('Reconnect')).register(self)

    def on_stomp_error(self, headers, message, error):
        """STOMP produced an error."""
        LOG.error('STOMP listener: Error:\n%s', message or error)

    def message(self, event, headers, message):
        """STOMP produced a message."""
        LOG.info('Message Received: %s', message)

    def disconnected(self, event, *args, **kwargs):
        # Wait a while then try to reconnect
        LOG.info('We got disconnected, reconnect')
        Timer(10, Event.create('reconnect')).register(self)

    def reconnect(self):
        """Try (re)connect to the STOMP server"""
        LOG.info('STOMP attempting to connect')
        self.fire(connect(host=self.host))


def main():
    logging.basicConfig()
    logging.getLogger().setLevel(logging.INFO)
    # Configure and run
    context = ssl.create_default_context()
    context.check_hostname = True
    context.verify_mode = ssl.CERT_REQUIRED

    # You can create an STOMP server to test for free at https://www.cloudamqp.com/
    uri = 'orangutan.rmq.cloudamqp.com'
    port = 61614
    login = 'xxxyyy'
    passcode = 'somepassword'
    host = 'xxxyyy'
    queue = 'test1'

    s = StompClient(uri, port, username=login, password=passcode, heartbeats=(10000, 10000), ssl_context=context)

    qr = QueueHandler(queue, host=host)
    s.register(qr)

    qr.run()


if __name__ == '__main__':
    main()



#!/usr/bin/env python
"""
Bridge Example

A Bridge example that demonstrates bidirectional parent/child
communications and displays the no. of events per second and latency.
"""

import sys
from signal import SIGINT, SIGTERM
from time import time
from traceback import format_exc

from circuits import Component, Event, handler, ipc


def log(msg, *args, **kwargs):
    sys.stderr.write('{:s}{:s}'.format(msg.format(*args), kwargs.get('n', '\n')))
    sys.stderr.flush()


def error(e):
    log('ERROR: {0:s}', e)
    log(format_exc())


def status(msg, *args):
    log('\r\x1b[K{0:s}', msg.format(*args), n='')


class ping(Event):
    """ping Event"""


class pong(Event):
    """pong Event"""


class Child(Component):
    def ping(self, ts):
        self.fire(ipc(pong(ts, time())))


class App(Component):
    def init(self):
        self.events = 0
        self.stime = time()

        Child().start(process=True, link=self)

    def ready(self, *args):
        self.fire(ipc(ping(time())))

    def pong(self, ts1, ts2):
        latency = (ts2 - ts1) * 1000.0
        status(
            f'{int(self.events / (time() - self.stime)):d} event/s @ {latency:0.2f}ms latency',
        )
        self.fire(ipc(ping(time())))

    def signal(self, signo, stack):
        if signo in [SIGINT, SIGTERM]:
            raise SystemExit(0)

    @handler()
    def on_event(self, *args, **kwargs):
        self.events += 1


App().run()



#!/usr/bin/env python
import os
from tempfile import mkstemp

from circuits.web import Controller

from .helpers import urlopen


class Root(Controller):
    def __init__(self, *args, **kwargs):
        super().__init__(self, *args, **kwargs)

        fd, self.filename = mkstemp()
        os.write(fd, b'Hello World!')
        os.close(fd)

    def __del__(self):
        os.remove(self.filename)

    def index(self):
        return self.serve_file(self.filename)


def test(webapp):
    f = urlopen(webapp.server.http.base)
    s = f.read()
    assert s == b'Hello World!'



#!/usr/bin/env python
"""
Simple Serial Example

This example shows how to use the ``circuits.io.Serial`` Component
to access serial data. This example simply echos back what it receives
on the serial port.

.. warning:: This example is currently untested.
"""

from circuits import Component, Debugger, handler
from circuits.io import Serial
from circuits.io.events import write


class EchoSerial(Component):
    def init(self, port):
        Serial(port).register(self)

    @handler('read')
    def on_read(self, data):
        """
        Read Event Handler

        This is fired by the underlying Serial Component when there has been
        new data read from the serial port.
        """
        self.fire(write(data))


# Start and "run" the system.
# Connect to /dev/ttyS0
app = EchoSerial('/dev/ttyS0')
Debugger().register(app)
app.run()



"""
Tools Test Suite

Test all functionality of the tools package.
"""

import pytest

from circuits import Component, reprhandler
from circuits.tools import findroot, inspect, kill, tryimport


class A(Component):
    def foo(self):
        print('A!')


class B(Component):
    def foo(self):
        print('B!')


class C(Component):
    def foo(self):
        print('C!')


class D(Component):
    def foo(self):
        print('D!')


class E(Component):
    def foo(self):
        print('E!')


class F(Component):
    def foo(self):
        print('F!')


def test_kill():
    a = A()
    b = B()
    c = C()
    d = D()
    e = E()
    f = F()

    a += b
    b += c

    e += f
    d += e
    a += d

    assert a.parent == a
    assert b.parent == a
    assert c.parent == b
    assert not c.components

    assert b in a.components
    assert d in a.components

    assert d.parent == a
    assert e.parent == d
    assert f.parent == e

    assert f in e.components
    assert e in d.components
    assert not f.components

    assert kill(d) is None
    while len(a):
        a.flush()

    assert a.parent == a
    assert b.parent == a
    assert c.parent == b
    assert not c.components

    assert b in a.components
    assert d not in a.components
    assert e not in d.components
    assert f not in e.components

    assert d.parent == d
    assert e.parent == e
    assert f.parent == f

    assert not d.components
    assert not e.components
    assert not f.components


def test_inspect():
    a = A()
    s = inspect(a)

    assert 'Components: 0' in s
    assert 'Event Handlers: 2' in s
    assert 'foo; 1' in s
    assert '<handler[*][foo] (A.foo)>' in s
    assert 'prepare_unregister_complete; 1' in s
    assert '<handler[<instance of A>][prepare_unregister_complete] (A._on_prepare_unregister_complete)>' in s


def test_findroot():
    a = A()
    b = B()
    c = C()

    a += b
    b += c

    root = findroot(a)
    assert root == a

    root = findroot(b)
    assert root == a

    root = findroot(c)
    assert root == a


def test_reprhandler():
    a = A()
    s = reprhandler(a.foo)
    assert s == '<handler[*][foo] (A.foo)>'

    pytest.raises(AttributeError, reprhandler, lambda: None)


def test_tryimport():
    import os

    m = tryimport('os')
    assert m is os


def test_tryimport_obj():
    from os import path

    m = tryimport('os', 'path')
    assert m is path


def test_tryimport_fail():
    m = tryimport('asdf')
    assert m is None



#!/usr/bin/env python
from circuits.web import Controller, expose

from .helpers import urlopen


class Root(Controller):
    @expose('test.txt')
    def index(self):
        return 'Hello world!'


class Leaf(Controller):
    channel = '/test'

    @expose('test.txt')
    def index(self, vpath=None):
        if vpath is None:
            return 'Hello world!'
        return 'Hello world! ' + vpath


def test(webapp):
    Leaf().register(webapp)

    f = urlopen(webapp.server.http.base + '/test.txt')
    s = f.read()
    assert s == b'Hello world!'

    f = urlopen(webapp.server.http.base + '/test/test.txt')
    s = f.read()
    assert s == b'Hello world!'
